@@ -1,13 +1,13 @@
 /*--------------------------------------------------------------------------
   ----- File:        subset.c 
   ----- Author:      Rainer Menzner (Rainer.Menzner@web.de)
-  ----- Date:        2002-12-07
+  ----- Date:        2005-05-07
   ----- Description: This file is part of t1lib. The program subset reads
                      a font file name and a string from the commandline
 		     and creates a subset of the font which is sufficient
 		     to display the string. It is an example of how to use
 		     font subsetting.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2002. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2005.
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -52,6 +52,11 @@ int main(int argc, char *argv[])
   unsigned long ofsize=0;
   FILE *ofp=stdout;
   
+  char** encoding     = 0;
+  char*  encodingfile = 0;
+  int    doreencode   = 0;
+  int    reresult     = 0;
+
   
   if (argc==1){
     printusage();
@@ -85,6 +90,18 @@ int main(int argc, char *argv[])
 	realargs -=1;
 	flags |=T1_SUBSET_SKIP_REENCODE;
       }
+      else if (strcmp( argv[j], "-e")==0) {
+	--realargs;
+	if ( j == argc-1 ) {
+	  fprintf(stderr, "subset: Option \"-e\": Missing filename argument.\n");
+	  printusage();
+	  return 1;
+	}
+	++j;
+	encodingfile=argv[j];
+	doreencode=1;
+	--realargs;
+      }
       else {
 	fprintf(stderr, "subset: Unknown option: %s\n", argv[j]);
 	printusage();
@@ -122,8 +139,17 @@ int main(int argc, char *argv[])
       return(1);
     }
   }
+
+  /* Load encoding file if specified */
+  if ( doreencode != 0 ) {
+    if ( (encoding = T1_LoadEncoding( encodingfile)) == NULL ) {
+      fprintf( stderr, "subset: Could not load Encoding File %s (T1_errno=%d, %s).\n",
+	       encodingfile, T1_errno, T1_StrError(T1_errno));
+      exit( 0);
+    }
+  }
   
-  
+
   /* First, build font data base */
   for (i=j; i<argc-1; i++){
     if ((result=T1_AddFont( argv[i]))<0)
@@ -155,6 +181,14 @@ int main(int argc, char *argv[])
     else {
       fprintf(stderr, "finished\n");
     }
+
+    /* Reencode font */
+    if ( doreencode != 0 ) {
+      if ( (reresult = T1_ReencodeFont( i, encoding)) != 0 ) {
+	fprintf( stderr, "Warning Reencoding font %d failed (%d)!\n", i, reresult);
+      }
+    }
+  
     fprintf( stderr, "Processing ... ");
     fflush(stderr);
     if ((subset=T1_SubsetFont(i, mask, flags, 64, 16384, &ofsize))==NULL) {
@@ -181,14 +215,21 @@ int main(int argc, char *argv[])
 
 void printusage( void)
 {
-  fprintf(stdout, "Usage: subset [-l|-a|-b|-f|-s] <fontfile1> [<fontfile2> ...] string\n");
+  fprintf(stdout, "Usage: subset [-l|-a|-b|-f|-s|-e <encfile>] <fontfile1> [<fontfile2> ...] string\n");
   fprintf(stdout, "\n");
-  fprintf(stdout, "Subset fontfile(s) according to `string' and write result to stdout (T1Lib-%s)!\n",
+  fprintf(stdout, "Subset source fontfile(s) according to `string' and write result\nto stdout (T1Lib-%s)!\n\n",
 	  T1_GetLibIdent());
-  fprintf(stdout, "Options: -l        Write a log-file t1lib.log\n");
-  fprintf(stdout, "         -a        Create ASCII-encrypted file (default)\n");
-  fprintf(stdout, "         -b        Create Binary-enrypted file\n");
-  fprintf(stdout, "         -f        Force reencoding of the font\n");
-  fprintf(stdout, "         -s        Skip reencoding of the font\n");
+  fprintf(stdout, "Options: -l                  Write a log-file t1lib.log.\n");
+  fprintf(stdout, "         -a                  Create ASCII-encrypted file (default).\n");
+  fprintf(stdout, "         -b                  Create Binary-enrypted file.\n");
+  fprintf(stdout, "         -f                  Force reencoding of the font subset, even if\n");
+  fprintf(stdout, "                             the source font uses internal StandardEncoding.\n");
+  fprintf(stdout, "         -s                  Skip reencoding of the font subset, even if the\n");
+  fprintf(stdout, "                             source font defines a font-specific encoding.\n");
+  fprintf(stdout, "         -e <encodingfile>   Load an encoding from specified file and reencode\n");
+  fprintf(stdout, "                             source font before starting to create the subset.\n");
+  fprintf(stdout, "                             This option also implies [-f].\n");
+
+  return;
 }
 
