@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1afmtool.c 
-  ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        04/18/1998
+  ----- Author:      Rainer Menzner (Rainer.Menzner@web.de)
+  ----- Date:        2005-05-01
   ----- Description: This file is part of the t1-library. It contains
                      functions for generating a fallback set of afm data
 		     from type 1 font files.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1998. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2005.
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -33,7 +33,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
+#if defined(_MSC_VER)
+# include <io.h>
+# include <sys/types.h>
+# include <sys/stat.h>
+#else
+# include <unistd.h>
+#endif
 #include <stdlib.h>
 #include <math.h>
 #include <time.h>
@@ -46,6 +52,7 @@
 #include "../type1/spaces.h"
 #include "../type1/util.h"
 #include "../type1/fontfcn.h"
+#include "../type1/paths.h"
 #include "../type1/regions.h"
 #include "../type1/blues.h"
 
@@ -66,6 +73,10 @@
 #define T1LIB_IDENT  "???.???"
 #endif
 
+
+extern char *t1_get_abort_message( int number);
+  
+
 /* T1_GenerateAFMFallbackInfo(): Generate fallback information from
    Type 1 font file by rasterizing every character at 1000 bp. Returns
    a pointer to a generated FontInfo struct or NULL in case of an error.
@@ -81,8 +92,19 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
   FontInfo *pAFMData;
   
   
+  /* We return to this if something goes wrong deep in the rasterizer */
+  if ((i=setjmp( stck_state))!=0) {
+    T1_errno=T1ERR_TYPE1_ABORT;
+    sprintf( err_warn_msg_buf, "t1_abort: Reason: %s",
+	     t1_get_abort_message( i));
+    T1_PrintLog( "T1_GenerateAFMFallbackInfo()", err_warn_msg_buf,
+	       T1LOG_ERROR);
+    return( NULL);
+  }
+  
+
   /* Check whether font is loaded: */
-  if (CheckForFontID(FontID)!=1){
+  if (T1_CheckForFontID(FontID)!=1){
     sprintf( err_warn_msg_buf,
 	     "Can't generate AFM Info from Font %d (invalid ID)\n", FontID);
     T1_PrintLog( "T1_GenerateAFMFallbackInfo()", err_warn_msg_buf,
@@ -131,7 +153,8 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
   charnames=T1_GetAllCharNames( FontID);
   /* and count number of characters */
   nochars=0;
-  while (charnames[nochars]!=NULL)
+  
+  while (charnames[nochars]!=NULL) 
     nochars++;
   pAFMData->numOfChars=nochars;
   /* Allocate memory for CharMetricInfo area */
@@ -153,7 +176,7 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
   /* Get metrics values */
   for (i=0; i<nochars; i++){
     area=fontfcnB_ByName( FontID, 0, S, charnames[i], &mode,
-			  pFontBase->pFontArray[FontID].pType1Data);
+			  pFontBase->pFontArray[FontID].pType1Data, DO_RASTER);
     
     if (area==NULL){
       sprintf( err_warn_msg_buf,
@@ -162,10 +185,12 @@ FontInfo *T1_GenerateAFMFallbackInfo( int FontID)
       T1_PrintLog( "T1_GenerateAFMFallbackInfo()", err_warn_msg_buf,
 		   T1LOG_WARNING);
       /* Return since we don't know how to fill the values */
-      for (j=i; j>=0; j--)
+      for (j=i-1; j>=0; j--)
 	free( pAFMData->cmi[j].name);
-      free( pAFMData->cmi);
-      free( pAFMData);
+      if (pAFMData->cmi!=NULL)
+	free( pAFMData->cmi);
+      if (pAFMData!=NULL)
+	free( pAFMData);
       T1_errno=mode;
       /* make sure to free S */
       if (S) {
@@ -248,18 +273,16 @@ int T1_WriteAFMFallbackFile( int FontID)
 
   int nochars;
   int capheight, ascender, xheight, descender;
-  int min, delta;
+  int min=0, delta;
   time_t s_clock, *tp;
   
   char *afmfilename;
   FILE *afmfile;
   CharMetricInfo **cmi;
   
-  extern psobj *StdEncArrayP;       /* For checking of a fonts encoding */
-  extern psobj *ISOLatin1EncArrayP; /* For checking of a fonts encoding */
-  
+
   /* Check for valid font */
-  if (CheckForFontID(FontID)!=1){
+  if (T1_CheckForFontID(FontID)!=1){
     sprintf( err_warn_msg_buf,
 	     "Warning: Invalid FontID, font %d not loaded!",
 	     FontID);
@@ -309,7 +332,7 @@ int T1_WriteAFMFallbackFile( int FontID)
   afmfilename[i+2]='f';
   afmfilename[i+3]='m';
   
-  if ((afmfile=fopen( afmfilename, "w"))==NULL){
+  if ((afmfile=fopen( afmfilename, "wb"))==NULL){
     sprintf( err_warn_msg_buf,
 	     "Could not open %s, (FontID = %d)",
 	     afmfilename, FontID);
@@ -357,7 +380,7 @@ int T1_WriteAFMFallbackFile( int FontID)
 	  afmfilename, pFontBase->pFontArray[FontID].pFontFileName,
 	  T1LIB_IDENT);
   fprintf(afmfile, "Comment File creation date: %s", ctime(&s_clock));
-  fprintf(afmfile, "Comment t1lib is copyright (c) Rainer Menzner, 1996, 1997, 1998.\n");
+  fprintf(afmfile, "Comment t1lib is copyright (c) Rainer Menzner, 1996-2001.\n");
   fprintf(afmfile, "Comment t1lib is distributed under the GNU General Public Library License (LGPL)\n");
   fprintf(afmfile, "FontName %s\n", T1_GetFontName( FontID));
   fprintf(afmfile, "FullName %s\n", T1_GetFullName( FontID));
@@ -379,14 +402,9 @@ int T1_WriteAFMFallbackFile( int FontID)
   fprintf(afmfile, "Notice %s\n", T1_GetNotice( FontID));
   /* Encoding */
   if (pFontBase->pFontArray[FontID].pFontEnc==NULL){
-    if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[ENCODING].value.data.arrayP
-	== StdEncArrayP){
+    if (pFontBase->pFontArray[FontID].info_flags & USES_STANDARD_ENCODING){
       fprintf( afmfile, "EncodingScheme AdobeStandardEncoding\n");
     }
-    else if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[ENCODING].value.data.arrayP
-	     == ISOLatin1EncArrayP){
-      fprintf( afmfile, "EncodingScheme ISOLatin1Encoding\n");
-    }
     else{
       fprintf( afmfile, "EncodingScheme FontSpecific\n");
     }
