@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1load.c 
   ----- Author:      Rainer Menzner (Rainer.Menzner@web.de)
-  ----- Date:        2002-11-27
+  ----- Date:        2005-05-01
   ----- Description: This file is part of the t1-library. It contains
                      functions for loading fonts  and for managing size
 		     dependent data.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2002. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2005. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -30,6 +30,15 @@
 
 #define ANSI_REALLOC_VM  
 
+/* Note: On some systems, like e.g. my Linux box, realloc() frequently returns
+         the identical pointer, if the memory chunk is *decreased* in size. As
+	 a consequence, pointer shifting (as implemented below) would never
+	 actually appear. The following definition enforces pointer shifting,
+	 and hence allows to check pointer shifting on every system. Do not
+	 activate this, it is meant for testing only!
+*/
+/* #define ANSI_REALLOC_ENFORCE_POINTERSHIFTING */
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -53,6 +62,7 @@
 #include "../type1/util.h"
 #include "../type1/fontfcn.h"
 #include "../type1/blues.h"
+#include "../type1/paths.h"
 #include "../type1/regions.h"
 
 
@@ -158,7 +168,12 @@ int T1_LoadFont( int FontID)
   /* Get size of VM, ... */
   tmp_size=((unsigned long)vm_used - (unsigned long)vm_base); 
   /* ... realloc to that size ... */
-  tmp_ptr=(char *)realloc(vm_base,  tmp_size); 
+#ifdef ANSI_REALLOC_ENFORCE_POINTERSHIFTING
+  tmp_ptr=(char *)malloc( tmp_size);
+  memcpy( tmp_ptr, vm_base, tmp_size);
+#else
+  tmp_ptr=(char *)realloc(vm_base,  tmp_size);
+#endif
   /* ... and shift all pointers refering to that area */
   if (tmp_ptr > vm_base){
     shift= (unsigned long)tmp_ptr - (unsigned long)vm_base;
@@ -211,24 +226,18 @@ int T1_LoadFont( int FontID)
       }
       /* The encoding needs special treatment: */
       if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_ENCODING){
-	/* If a builtin encoding is used, it is sufficient to shift the pointer
-	   to the Encoding since the character-namestrings of builtin encodings
-	   are static and thus located on the heap.
-	   For font-specific encoding, character-namestrings reside in VM and
-	   thus each entry has to be shifted. 
-	   Caution: We still have to shift the builtin encoding-pointer, since
-	   they also point to are located in VM: */
-	ldummy=(long)StdEncArrayP;
-	ldummy +=shift;
-	StdEncArrayP=(psobj *)ldummy;
-	ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP;
-	ldummy +=shift;
-	pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP=(char *)ldummy;
+	/* If builtin StandardEncoding is used, we do nothing here. Standard Encoding
+	   is now located once for all fonts on the heap. For font-specific encodings
+	   we have to move all pointers appropriately, because this is entirely located
+	   in VM */
 	if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP
-	    == StdEncArrayP){ /* Font uses builtin standard encoding */
+	    == StdEncArrayP){ /* Font uses builtin StandardEncoding */
 	  ;
 	} 
 	else{ /* Font-specific encoding */ 
+	  ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP;
+	  ldummy +=shift;
+	  pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP=(char *)ldummy;
 	  for (k=0; k<256; k++){
 	    ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP;
 	    /* The ".notdef" is also static and may not be shifted (Thanks, Derek ;) */
@@ -346,24 +355,18 @@ int T1_LoadFont( int FontID)
       }
       /* The encoding needs special treatment: */
       if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.type==OBJ_ENCODING){
-	/* If a builtin encoding is used, it is sufficient to shift the pointer
-	   to the Encoding since the character-namestrings of builtin encodings
-	   are static and thus located on the heap.
-	   For font-specific encoding, character-namestrings reside in VM and
-	   thus each entry has to be shifted. 
-	   Caution: We still have to shift the builtin encoding-pointer, since
-	   they also point to are located in VM: */
-	ldummy=(long)StdEncArrayP;
-	ldummy -=shift;
-	StdEncArrayP=(psobj *)ldummy;
-	ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP;
-	ldummy -=shift;
-	pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP=(char *)ldummy;
+	/* If builtin StandardEncoding is used, we do nothing here. Standard Encoding
+	   is now located once for all fonts on the heap. For font-specific encodings
+	   we have to move all pointers appropriately, because this is entirely located
+	   in VM */
 	if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP
-	     == StdEncArrayP){ /* Font uses builtin encoding */
+	    == StdEncArrayP){ /* Font uses builtin StandardEncoding */
 	  ;
 	} 
 	else{ /* Font-specific encoding */ 
+	  ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP;
+	  ldummy -=shift;
+	  pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.valueP=(char *)ldummy;
 	  for (k=0; k<256; k++){
 	    ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP;
 	    /* The ".notdef" is also static and may not be shifted (Thanks, Derek ;) */
@@ -436,6 +439,11 @@ int T1_LoadFont( int FontID)
 	     (unsigned long)vm_base);
     T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
   }
+#ifdef ANSI_REALLOC_ENFORCE_POINTERSHIFTING
+  /* If pointer shifting had been enforced by allocating from
+     scratch using malloc() free the previous vm. */
+  free( vm_base);
+#endif
 #endif
     
   /* Generate a message how much VM the current font consumes */
