@@ -82,8 +82,8 @@ STATIC unsigned char TheBuffer[F_BUFSIZ];
  
 /* Our routines */
 F_FILE *T1Open(), *T1eexec();
-int T1Close();
-int T1Read(), T1Getc(), T1Ungetc();
+int T1Close(F_FILE *);
+int T1Read(), T1Getc(F_FILE *), T1Ungetc(int,F_FILE *);
 void T1io_reset(void);
 STATIC int T1Decrypt(), T1Fill();
  
@@ -277,6 +277,7 @@ int T1GetTrailer(char *string,
   char *ctmP;
   int i=0, j;
   int datasize;
+  int len;
   
   datasize=size;
   
@@ -297,11 +298,18 @@ int T1GetTrailer(char *string,
       datasize=i; /* we skip the segment marker of pfb-files */
     }
     if ((ctmP=strstr( &(buf[j]), "cleartomark"))!=NULL) {
-      memcpy( string, &(buf[i]), datasize-i);
-      string[datasize-i]='\0';
+      /* buf[i-1] now is the first character after cleartomark. Advance now
+	 to the next non white character of EOF. */
+      len = datasize - i;
+      while ( (isspace( (int)(buf[i-1])) != 0) &&
+	      (i < datasize) ) {
+	++i;
+      }
+      memcpy( string, &(buf[i-1]), len);
+      string[len]='\0';
       lseek( f->fd, off_save, SEEK_SET);
       free( buf);
-      return( datasize-i);
+      return len;
     }
     i--;
   }
